#ifndef CAH_H
#define CAH_H

#include <QString>
#include <QVector>
#include <QList>
#include <QStringList>
#include <QDebug>

enum {DST_EUCLI = 0, DST_EUCLI2 };
enum {CRI_MIN = 0, CRI_MAX, CRI_MOY};

class Item : public QVector<double> {
private:
	uint  id;
public:
	Item(uint id) {
		this->id = id;
	}

	QString getName() {
		return QString("a%1").arg(this->id);
	}

	uint getId() {
		return id;
	}
};

class Cluster : public QList<Item> {
private:
        uint id;
	double dst;
	QList<Cluster> childs;
public:
	Cluster(uint id, double dst) {
		this->id = id;
		this->dst = dst;
	}

	double getDst() {
		return this->dst;
	}

	QString getName() {
		return QString("A%1").arg(this->id);
	}

	uint getId() {
		return this->id;
	}

	QString getValues() {
		QString str;
		for(int i = 0; i < this->size(); ++i) {
			str.append(((*this)[i]).getName() + " ");
		}
		return str;
	}

	void addChild(const Cluster & child) {
		this->childs.append(child);
	}

	QList<Cluster> getChilds() {
		return childs;
	}
};

class CAH {
private:
	QList<Item> items;
	QList<Cluster> clusters;
	QVector<double> pond;
	uint ic;
	uint step;
	uint cri;
	uint dst;
	uint prec;

	void loadData(const QString & data);
	double calccri(const Cluster & a, const Cluster & b);
	double calcdst(const Item & a, const Item & b);

public:
	CAH(QString data, QVector<double> pond, uint dst, uint cri, uint prec);
	QString getData();
	QString run();
	QList<Cluster> getClusters();
};

#endif
